//*****************************************************************************
//
// grlib_demo.c - Demonstration of the TivaWare Graphics Library.
//
// Copyright (c) 2008-2013 Texas Instruments Incorporated.  All rights reserved.
// Software License Agreement
// 
// Texas Instruments (TI) is supplying this software for use solely and
// exclusively on TI's microcontroller products. The software is owned by
// TI and/or its suppliers, and is protected under applicable copyright
// laws. You may not combine this software with "viral" open-source
// software in order to form a larger program.
// 
// THIS SOFTWARE IS PROVIDED "AS IS" AND WITH ALL FAULTS.
// NO WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT
// NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. TI SHALL NOT, UNDER ANY
// CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
// DAMAGES, FOR ANY REASON WHATSOEVER.
// 
// This is part of revision 2.0.1.11577 of the DK-TM4C123G Firmware Package.
//
//*****************************************************************************

/*
 * Project target TM4C123GH6PM (256KB Flash, 32KB SRAM, 64LQFP) in Tiva C LaunchPad
 * Under Advanced Options -> Predefined Symbols-> PART_TM4C123GH6PM
 * Use startup_ccs.c instead of the default tm4c123gh6pm_startup_ccs.c in original version
 * However, it doesn't make any difference even inc/hw_nvic.h and inc/hw_types.h removed
 * #include "drivers/cfal96x64x16.h" removed and cfal96x64x16.c removed from project
 * Somehow, optimization set to none in order to view variables...
 *
 * */

//#define USE_VIBRATION

#include <stdint.h>
#include <stdbool.h>
#include "inc/hw_memmap.h"
#include "inc/hw_gpio.h"
#include "inc/hw_sysctl.h"

#include "driverlib/sysctl.h"
#include "driverlib/gpio.h"
#include "driverlib/pwm.h"
#include "driverlib/pin_map.h"
#include "driverlib/rom.h"
#include "driverlib/debug.h"

#include "SSD2805_S6D04D2.h"	//Low level driver for S6D04D2 with MIPI bridge SSD2805
#include "grlib/grlib.h"


//*****************************************************************************
//
//! \addtogroup example_list
//! <h1>Graphics Library Demonstration (grlib_demo)</h1>
//!
//! This application provides a demonstration of the capabilities of the
//! TivaWare Graphics Library.  The display will be configured to demonstrate
//! the available drawing primitives: lines, circles, rectangles, strings, and
//! images.
//
//*****************************************************************************

//*****************************************************************************
//
// Graphics context used to show text on the CSTN display.
//
//*****************************************************************************
tContext g_sContext;

extern const uint8_t color_balls[];
extern const uint8_t SIDlogo[];
extern const uint8_t Grandma114x116[];
extern const uint8_t number0[];
extern const uint8_t number1[];
extern const uint8_t number2[];
extern const uint8_t number3[];
extern const uint8_t number4[];
extern const uint8_t number5[];
extern const uint8_t number6[];
extern const uint8_t number7[];
extern const uint8_t number8[];
extern const uint8_t number9[];
extern const uint8_t colon[];
extern const uint8_t android_icon[];
extern const uint8_t message_icon[];
extern const uint8_t angrybird_icon[];

//*****************************************************************************
//
// The error routine that is called if the driver library encounters an error.
//
//*****************************************************************************
#ifdef DEBUG
void
__error__(char *pcFilename, uint32_t ui32Line)
{
}
#endif

//*****************************************************************************
//
// A simple demonstration of the features of the TivaWare Graphics Library.
//
//*****************************************************************************
int
main(void)
{
    uint32_t ui32Idx;
    tRectangle sRect;
    uint16_t x,y;

    //
    // Enable lazy stacking for interrupt handlers.  This allows floating-point
    // instructions to be used within interrupt handlers, but at the expense of
    // extra stack usage.
    //
    ROM_FPULazyStackingEnable();

    //
    // Set the clocking to run from the PLL.
    //
    ROM_SysCtlClockSet(SYSCTL_SYSDIV_2_5 | SYSCTL_USE_PLL | SYSCTL_XTAL_16MHZ | SYSCTL_OSC_MAIN);
	//Low level TX_CLK output for 20MHz system clock generated by PWM assumed the system clock is 80MHz
	ROM_SysCtlPWMClockSet(SYSCTL_PWMDIV_1);	//set PWM clock to be system clock / 1, 2, 4, 8, 16, 32, or 64

/*
 *
    uint32_t ui32PWMClock, ui32Load;
	ui32PWMClock = SysCtlClockGet();

	ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_PWM1);			//PWM module 1 (PF3) for MIPI_CLOCK
	ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOF);
	ROM_GPIOPinTypePWM(GPIO_PORTF_BASE, GPIO_PIN_3);
	ROM_GPIOPinConfigure(GPIO_PF3_M1PWM7);
	ROM_PWMGenConfigure(PWM1_BASE, PWM_GEN_3, PWM_GEN_MODE_DOWN);
	ui32Load = (ui32PWMClock / 20000000UL);
	ROM_PWMGenPeriodSet(PWM1_BASE, PWM_GEN_3, ui32Load);
	ROM_PWMPulseWidthSet(PWM1_BASE, PWM_OUT_7, ui32Load/2);	//duty cycle 50%
	ROM_PWMOutputState(PWM1_BASE, PWM_OUT_7_BIT, true);		//enable PWM7 output
*/

/*
	ui32PWMClock = ROM_SysCtlClockGet();

	ROM_SysCtlPeripheralEnable(LCD_TXCLK_PWMMOD);
	ROM_SysCtlPeripheralEnable(LCD_TXCLK_PERIPH);
	ROM_GPIOPinTypePWM(LCD_TXCLK_BASE, LCD_TXCLK_PIN);
	ROM_GPIOPinConfigure(LCD_TXCLK_PIN_MUX);
	ROM_PWMGenConfigure(LCD_TXCLK_PWM_BASE, LCD_TXCLK_PWM_GEN, PWM_GEN_MODE_DOWN);
	ROM_PWMGenPeriodSet(LCD_TXCLK_PWM_BASE, LCD_TXCLK_PWM_GEN, (ui32PWMClock / LCD_TXCLK_FREQ));
	ROM_PWMPulseWidthSet(LCD_TXCLK_PWM_BASE, LCD_TXCLK_PWM_OUT, (ui32PWMClock / LCD_TXCLK_FREQ)/2);	//duty cycle 50%
	ROM_PWMOutputState(LCD_TXCLK_PWM_BASE, LCD_TXCLK_PWM_BIT, true);		//enable PWM7 output
	//Now it is time to enable all PWM channels
	ROM_PWMGenEnable(PWM1_BASE, PWM_GEN_3);	//MIPI_CLOCK, BT_CLOCK & PWM for backlight
	//now the system clock of 20MHz generated
*/
    DisplayReset();


#if defined SSD1268_H_
    GrContextInit(&g_sContext, &g_DisplaySolomonSSD1268);
    GrContextBackgroundSet(&g_sContext, ClrBlack);
#elif defined SSD2805_S6D04D2_H_
    GrContextInit(&g_sContext, &g_DisplaySolomonSSD2805);
    GrContextBackgroundSet(&g_sContext, ClrBlack);
#endif

    //debug vibration motor
#ifdef USE_VIBRATION
    DisplayVibrationConfig();
    DisplayVibrationOff();
#endif

    while(1)
    {
    DisplayRectFill(0, 0, DISP_HOR_RESOLUTION-1, DISP_HOR_RESOLUTION-1, ClrBlack);

    //rectangle ranges from (4,4) to (235,115) inclusive (L1)
    sRect.i16XMin = 4;
    sRect.i16YMin = 4;
    sRect.i16XMax = 235;
    sRect.i16YMax = 115;
    //GrContextForegroundSet(&g_sContext, 0x000050ed);
    //GrRectFill(&g_sContext, &sRect);
    DisplayRectFill(sRect.i16XMin, sRect.i16YMin, sRect.i16XMax, sRect.i16YMax, 0x000050ed);

    //rectangle ranges from (4,120) to (58,175) inclusive (L2)
    sRect.i16XMin = 4;
    sRect.i16YMin = 120;
    sRect.i16XMax = 58;
    sRect.i16YMax = 175;
    //GrContextForegroundSet(&g_sContext, 0x00e1016f);
    //GrRectFill(&g_sContext, &sRect);
    DisplayRectFill(sRect.i16XMin, sRect.i16YMin, sRect.i16XMax, sRect.i16YMax, 0x00e1016f);
    DisplayBackgroundSet(0x00e1016f);
    DisplayPutImage((4+3), (120+4), angrybird_icon);

    //L3
    sRect.i16XMin = 63;
    sRect.i16YMin = 120;
    sRect.i16XMax = 117;
    sRect.i16YMax = 175;
    //GrContextForegroundSet(&g_sContext, 0x00fa9900);
    //GrRectFill(&g_sContext, &sRect);
    DisplayRectFill(sRect.i16XMin, sRect.i16YMin, sRect.i16XMax, sRect.i16YMax, 0x00fa9900);
    DisplayBackgroundSet(0x00fa9900);
    DisplayPutImage((63+3), (120+4), android_icon);

    //L4
    sRect.i16XMin = 122;
    sRect.i16YMin = 120;
    sRect.i16XMax = 235;
    sRect.i16YMax = 175;
    //GrContextForegroundSet(&g_sContext, 0x02c3f4);
    //GrRectFill(&g_sContext, &sRect);
    DisplayRectFill(sRect.i16XMin, sRect.i16YMin, sRect.i16XMax, sRect.i16YMax, 0x02c3f4);


    //L5
    sRect.i16XMin = 4;
    sRect.i16YMin = 180;
    sRect.i16XMax = 117;
    sRect.i16YMax = 235;
    //GrContextForegroundSet(&g_sContext, ClrGreen);
    //GrRectFill(&g_sContext, &sRect);
    DisplayRectFill(sRect.i16XMin, sRect.i16YMin, sRect.i16XMax, sRect.i16YMax, 0x057622);

    //L6
    sRect.i16XMin = 122;
    sRect.i16YMin = 180;
    sRect.i16XMax = 176;
    sRect.i16YMax = 235;
    DisplayRectFill(sRect.i16XMin, sRect.i16YMin, sRect.i16XMax, sRect.i16YMax, 0x80d209);

    //L7
    sRect.i16XMin = 180;
    sRect.i16YMin = 180;
    sRect.i16XMax = 235;
    sRect.i16YMax = 235;
    DisplayRectFill(sRect.i16XMin, sRect.i16YMin, sRect.i16XMax, sRect.i16YMax, 0xe1016f);

    //display a digital clock demo
		#define CLOCK_LEFT	12
		#define CLOCK_TOP	20
		#define CLOCK_DIGIT_OFFSET	34

    DisplayBackgroundSet(0x000050ed);
    DisplayPutImage(CLOCK_LEFT,CLOCK_TOP, number1);
    DisplayPutImage(CLOCK_LEFT+CLOCK_DIGIT_OFFSET, CLOCK_TOP, number0);
    DisplayPutImage(CLOCK_LEFT+(CLOCK_DIGIT_OFFSET*2), CLOCK_TOP, colon);
    DisplayPutImage(CLOCK_LEFT+(CLOCK_DIGIT_OFFSET*3), CLOCK_TOP, number2);
    DisplayPutImage(CLOCK_LEFT+(CLOCK_DIGIT_OFFSET*4), CLOCK_TOP, number0);
    DelayMs(1000);

    uint8_t MessageFlag=false, MessageIcon=false;

    for(x=1; x<10; x++)
    {
    	DisplayBackgroundSet(0x000050ed);
    	switch(x)
    	{
    	case 0:
            DisplayPutImage(CLOCK_LEFT+(CLOCK_DIGIT_OFFSET*4), CLOCK_TOP, number0);
            break;
    	case 1:
            DisplayPutImage(CLOCK_LEFT+(CLOCK_DIGIT_OFFSET*4), CLOCK_TOP, number1);
            break;
    	case 2:
            DisplayPutImage(CLOCK_LEFT+(CLOCK_DIGIT_OFFSET*4), CLOCK_TOP, number2);
            break;
    	case 3:
    		if(MessageFlag==false)
    		{
    			DisplayPutImage(122, 120, Grandma114x116);
    			MessageFlag=true;
    		}
            DisplayPutImage(CLOCK_LEFT+(CLOCK_DIGIT_OFFSET*4), CLOCK_TOP, number3);
            break;
    	case 4:
            DisplayPutImage(CLOCK_LEFT+(CLOCK_DIGIT_OFFSET*4), CLOCK_TOP, number4);
            break;
    	case 5:
            DisplayPutImage(CLOCK_LEFT+(CLOCK_DIGIT_OFFSET*4), CLOCK_TOP, number5);
            break;
    	case 6:
            DisplayPutImage(CLOCK_LEFT+(CLOCK_DIGIT_OFFSET*4), CLOCK_TOP, number6);
            break;
    	case 7:
            DisplayPutImage(CLOCK_LEFT+(CLOCK_DIGIT_OFFSET*4), CLOCK_TOP, number7);
            break;
    	case 8:
            DisplayPutImage(CLOCK_LEFT+(CLOCK_DIGIT_OFFSET*4), CLOCK_TOP, number8);
            break;
    	case 9:
            DisplayPutImage(CLOCK_LEFT+(CLOCK_DIGIT_OFFSET*4), CLOCK_TOP, number9);
            break;

    	}

        DelayMs(1000);

        if(MessageFlag==true)
        {
        	if(MessageIcon==false)
        	{
        		DisplayBackgroundSet(0x057622);
        		DisplayPutImage(4+33, 180+4, message_icon);
#ifdef USE_VIBRATION
        		DisplayVibrationOn();
#endif
        		MessageIcon=true;
        	}
        	else
        	{
        		DisplayRectFill(4, 180, 117, 235, 0x057622);
#ifdef USE_VIBRATION
        		DisplayVibrationOff();
#endif
        		MessageIcon=false;
        	}
        }
    }
    DelayMs(1000);

    //make sure vibration motor is off at the end
    DisplayRectFill(4, 180, 117, 235, 0x057622);
#ifdef USE_VIBRATION
    DisplayVibrationOff();
#endif

    DelayMs(2000);
    DisplayRectFill(0, 0, DISP_HOR_RESOLUTION-1, DISP_VER_RESOLUTION-1, ClrRed);
    DelayMs(2000);
    DisplayRectFill(0, 0, DISP_HOR_RESOLUTION-1, DISP_VER_RESOLUTION-1, ClrGreen);
    DelayMs(2000);
    DisplayRectFill(0, 0, DISP_HOR_RESOLUTION-1, DISP_VER_RESOLUTION-1, ClrBlue);
    DelayMs(2000);

    DisplayPutImage(0,0, color_balls);

    DelayMs(2000);
    DisplayRectFill(0, 0, DISP_HOR_RESOLUTION-1, DISP_VER_RESOLUTION-1, ClrWhite);
    DelayMs(2000);


    //
    // Initialize the display driver.
    //
    //CFAL96x64x16Init();

    //
    // Initialize the graphics context.
    //
    // GrContextInit(&g_sContext, &g_sCFAL96x64x16);
    //
    // Fill the top 12 rows of the screen with blue to create the banner.
    //

    //GrContextInit(&g_sContext, &g_DisplaySolomonSSD1268);

    sRect.i16XMin = 0;
    sRect.i16YMin = 0;
    sRect.i16XMax = GrContextDpyWidthGet(&g_sContext) - 1;
    sRect.i16YMax = 11;
    GrContextForegroundSet(&g_sContext, ClrDarkBlue);
    GrRectFill(&g_sContext, &sRect);

    //
    // Put a white box around the banner.
    //
    GrContextForegroundSet(&g_sContext, ClrWhite);
    GrRectDraw(&g_sContext, &sRect);

    //
    // Put the application name in the middle of the banner.
    //
   // GrContextFontSet(&g_sContext, g_psFontFixed6x8);
    GrContextFontSet(&g_sContext, g_psFontCmtt48);

    GrStringDrawCentered(&g_sContext, "grlib_demo", -1,
                         GrContextDpyWidthGet(&g_sContext) / 2, 5, 0);

    //
    // Draw a vertical sweep of lines from red to green.
    //
    for(ui32Idx = 0; ui32Idx <= 8; ui32Idx++)
    {
        GrContextForegroundSet(&g_sContext,
                               (((((10 - ui32Idx) * 255) / 8) << ClrRedShift) |
                                (((ui32Idx * 255) / 8) << ClrGreenShift)));
        GrLineDraw(&g_sContext, 60, 60, 0, 60 - (5 * ui32Idx));
    }

    //
    // Draw a horizontal sweep of lines from green to blue.
    //
    for(ui32Idx = 1; ui32Idx <= 11; ui32Idx++)
    {
        GrContextForegroundSet(&g_sContext,
                               (((((11 - ui32Idx) * 255) / 11) <<
                                 ClrGreenShift) |
                                (((ui32Idx * 255) / 11) << ClrBlueShift)));
        GrLineDraw(&g_sContext, 60, 60, (ui32Idx * 5), 20);
    }

    //
    // Draw a filled circle with an overlapping circle.
    //
    GrContextForegroundSet(&g_sContext, ClrBlue);
    GrCircleFill(&g_sContext, 80, 30, 15);
    GrContextForegroundSet(&g_sContext, ClrWhite);
    GrCircleDraw(&g_sContext, 80, 30, 15);

    //
    // Draw a filled rectangle with an overlapping rectangle.
    //
    GrContextForegroundSet(&g_sContext, ClrGray);
    sRect.i16XMin = 8;
    sRect.i16YMin = 45;
    sRect.i16XMax = 46;
    sRect.i16YMax = 51;
    GrRectFill(&g_sContext, &sRect);
    GrContextForegroundSet(&g_sContext, ClrWhite);
    sRect.i16XMin += 4;
    sRect.i16YMin += 4;
    sRect.i16XMax += 4;
    sRect.i16YMax += 4;
    GrRectDraw(&g_sContext, &sRect);

    //
    // Draw a piece of text in fonts of increasing size.
    //
    GrContextForegroundSet(&g_sContext, ClrBlack);
    GrStringDraw(&g_sContext, "Strings", -1, 6, 16, 0);
    GrContextForegroundSet(&g_sContext, ClrSilver);
    GrStringDraw(&g_sContext, "Strings", -1, 7, 17, 0);

    //
    // Draw an image.
    //
    //GrTransparentImageDraw(&g_sContext, g_pui8Logo, 64, 34, ClrBlack);

    GrImageDraw(&g_sContext, SIDlogo, 100,0);
    //GrImageDraw(&g_sContext, g_pui8Logo, 64, 34);
    //GrImageDraw(&g_sContext, g_Photo8BPP, 0,0);
    //
    // Flush any cached drawing operations.
    //
    GrFlush(&g_sContext);

    }
}
